var assert = require('assert')
var vows = require('vows')

var fire = require('fire')
var fireTest = require('../index.js')
var fireAssert = require('fire-assert')
var path = require('path')

var TestContextType = fireTest.TestContextType
var TestStepType = fireTest.TestStepType
function createRuntime(expressions, testContext) {
	var runtime = new fire.Runtime()
	runtime.moduleRequire = function(moduleName) {
		return require(moduleName)
	}
	runtime.setBaseDir(path.dirname(module.filename))
	runtime.loadModuleInstance(fireTest,'fire-test')
	expressions.forEach(function(expDef){
		runtime.registerWellKnownExpressionDefinition(expDef);
	})
	runtime.load(function(initErr) {
		if(initErr) {
			testContext.callback(initErr, {
				runtime: runtime,
				error: initErr,
				result: null
			})
		}
		var expDef = runtime.getExpressionDefinition(expressions[0].name)
		var exp = new(expDef.implementation)
		exp.errorCallback = function(err) {
			testContext.callback(null, {
				runtime: runtime,
				error: err,
				result: null
			})
		}
		exp.loopCallback = function() {
			throw "Loop callback catched!"
		}
		exp.resultCallback = function(res) {
			testContext.callback(null, {
				runtime: runtime,
				error: null,
				result: res
			})
		}
		exp.runtime = runtime
		exp.run()
	})
	return runtime
}

vows.describe('fire-test').addBatch({
	"When I execute @Test.Context.Run with no test available in the runtime": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				}], this)
		}
		,"it should return a test context with no sub-contexts": function(err, res){
			assert.isNull(res.error)
			assert.isNotNull(res.result)
			assert.instanceOf(res.result.contexts, Array)
			assert.lengthOf(res.result.contexts, 0)
			assert.equal(res.result.type, TestContextType.Root)
		}
		,"and the context should be of type Root": function(err, res){
			assert.equal(res.result.type, TestContextType.Root)
		}
	},
	"When I run the tests with a scenario": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							
						}
					}
				}], this)
		}
		,"it should return an object with a sub-context for the scenario": function(err, res){
			assert.isNull(res.error)
			assert.isNotNull(res.result)
			assert.instanceOf(res.result.contexts, Array)
			assert.lengthOf(res.result.contexts, 1)
			assert.equal(res.result.contexts[0].description, "Signup")
		}
		,"the sub context must be of Scenario type": function(err, res){
			assert.equal(res.result.contexts[0].type, TestContextType.Scenario)
		}
	}
	,"When I run the tests with a scenario and a nested @Test.When": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.When(I signup with valid information)": {
								
							}
						}
					}
				}], this)
		}
		,"the When context should be of type Step": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
		}
		,"the When context should be of StepType When": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].stepType, TestStepType.When)
		}
		,"the When context description should be the given in the hint of the @Test.When expression": function(err, res){
			assert.equal(res.result.contexts[0].contexts[0].description, "I signup with valid information")
		}
	}
	,"When I use @Test.Topic": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.When(I signup with valid information)": {
								"@Test.Topic": {
									"desc": "This is the topic of the Test"
								}
							}
						}
					}
				}], this)
		}
		,"it should change the topic of the current test context": function(err, res){
			assert.deepEqual(res.result.contexts[0].contexts[0].topic, {
				"desc": "This is the topic of the Test"
			})
		}
	}
	,"When I run the tests with a scenario and a nested @Test.Then": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.Then(I should get 200 OK)": {
								
							}
						}
					}
				}], this)
		}
		,"the Then context should be of type Step": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
		}
		,"the Then context should be of StepType Then": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].stepType, TestStepType.Then)
		}
		,"the Then context description should be the given in the hint of the @Test.Then expression": function(err, res){
			assert.equal(res.result.contexts[0].contexts[0].description, "I should get 200 OK")
		}
	}
	,"When I run the tests with a scenario and a nested @Test.And": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.And(the email should match)": {
								
							}
						}
					}
				}], this)
		}
		,"the And context should be of type Step": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
		}
		,"the And context should be of StepType And": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].stepType, TestStepType.And)
		}
		,"the And context description should be the given in the hint of the @Test.And expression": function(err, res){
			assert.equal(res.result.contexts[0].contexts[0].description, "the email should match")
		}
	}
	,"When I use an assertion inside one Step context like @Assert.Then and the assertion does not fail": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.When(I provide a valid email address)": {
								"@Test.Topic": "chuck@example.com",
								"@Test.Then(The validation should succeed)": {
									"@Assert.Email": null
								}
							}
						}
					}
				}], this)
		}
		,"there should not be errors": function(err, res){
			assert.isNull(res.error)
		}
		,"the Assert step should have no errors": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].contexts[0].stepType, TestStepType.Then)
			assert.isNull(res.result.contexts[0].contexts[0].contexts[0].error)
		}
	}
	,"When I use an assertion inside one Step context like @Assert.Then and the assertion fails": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@Test.Context.Run": null
					}
				},
				{
					"name": "testSomething",
					"test": true,
					"json": {
						"@Test.Scenario(Signup)": {
							"@Test.When(I provide a valid email address)": {
								"@Test.Topic": "notValidAddress",
								"@Test.Then(The validation should succeed)": {
									"@Assert.Email": null
								}
							}
						}
					}
				}], this)
		}
		,"there should not be errors": function(err, res){
			assert.isNull(res.error)
		}
		,"the Assert step should have the assertion error": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].contexts[0].stepType, TestStepType.Then)
			assert.isNotNull(res.result.contexts[0].contexts[0].contexts[0].error)
			assert.instanceOf(res.result.contexts[0].contexts[0].contexts[0].error, fireAssert.AssertError)
			assert.equal(res.result.contexts[0].contexts[0].contexts[0].error.message, "Not a valid Email address")
		}
		,"the Parent Step should have failed": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].contexts[0].type, TestContextType.Step)
			assert.equal(res.result.contexts[0].contexts[0].stepType, TestStepType.When)
			assert.equal(res.result.contexts[0].contexts[0].description, "I provide a valid email address")
			assert.isTrue(res.result.contexts[0].contexts[0].error)
		}
		,"the Scenario should have failed": function(err, res){
			assert.lengthOf(res.result.contexts[0].contexts,1)
			assert.equal(res.result.contexts[0].type, TestContextType.Scenario)
			assert.isTrue(res.result.contexts[0].error)
		}
	}
}).export(module)